Assembler

Struct Assembler 

pub struct Assembler { /* private fields */ }
PVM program assembler.

Implementations

Source

impl Assembler

pub fn new() -> Self

pub fn add_jump_entry(&mut self) -> usize

Add a jump table entry pointing to the current code offset. Returns the jump table index.

pub fn add_jump_entry_at(&mut self, offset: u32) -> usize

Add a jump table entry pointing to a specific code offset.

pub fn current_offset(&self) -> u32

Get the current code offset.

pub fn label(&mut self, name: &str) -> &mut Self

Define a label at the current code position.

pub fn trap(&mut self) -> &mut Self

Opcode 0: Trap (halt with error)

pub fn fallthrough(&mut self) -> &mut Self

Opcode 1: Fallthrough (nop, continue to next instruction)

pub fn ecalli(&mut self, id: u32) -> &mut Self

Opcode 10: ecalli (host call with immediate ID)

pub fn load_imm_64(&mut self, rd: Reg, imm: u64) -> &mut Self

Opcode 20: load_imm_64 (load 64-bit immediate into register)

pub fn jump(&mut self, target: u32) -> &mut Self

Opcode 40: jump (unconditional jump to offset)

pub fn jump_ind(&mut self, rd: Reg, imm: u32) -> &mut Self

Opcode 50: jump_ind (indirect jump through register + immediate)

pub fn load_imm(&mut self, rd: Reg, imm: i32) -> &mut Self

Opcode 51: load_imm (load sign-extended immediate into register)

pub fn load_u8(&mut self, rd: Reg, addr: u32) -> &mut Self

Opcode 52: load_u8 (load u8 from address in immediate)

pub fn load_u64(&mut self, rd: Reg, addr: u32) -> &mut Self

Opcode 58: load_u64 (load u64 from address in immediate)

pub fn store_u8(&mut self, rd: Reg, addr: u32) -> &mut Self

Opcode 59: store_u8 (store u8 from register to address)

pub fn store_u64(&mut self, rd: Reg, addr: u32) -> &mut Self

Opcode 62: store_u64 (store u64 from register to address)

pub fn load_imm_jump(&mut self, rd: Reg, imm: i32, target: u32) -> &mut Self

Opcode 80: load_imm_jump (load immediate into register and jump)

pub fn branch_eq_imm(&mut self, rd: Reg, imm: i32, target: u32) -> &mut Self

Opcode 81: branch_eq_imm (branch if register == immediate)

pub fn branch_ne_imm(&mut self, rd: Reg, imm: i32, target: u32) -> &mut Self

Opcode 82: branch_ne_imm (branch if register != immediate)

pub fn branch_lt_u_imm(&mut self, rd: Reg, imm: i32, target: u32) -> &mut Self

Opcode 83: branch_lt_u_imm (branch if register < unsigned immediate)

pub fn move_reg(&mut self, rd: Reg, ra: Reg) -> &mut Self

Opcode 100: move_reg (copy register)

pub fn load_ind_u8(&mut self, rd: Reg, ra: Reg, imm: i32) -> &mut Self

Opcode 124: load_ind_u8 (load u8 from [rA + imm] into rD)

pub fn load_ind_u32(&mut self, rd: Reg, ra: Reg, imm: i32) -> &mut Self

Opcode 128: load_ind_u32 (load u32 from [rA + imm] into rD)

pub fn load_ind_u64(&mut self, rd: Reg, ra: Reg, imm: i32) -> &mut Self

Opcode 130: load_ind_u64 (load u64 from [rA + imm] into rD)

pub fn store_ind_u8(&mut self, rd: Reg, ra: Reg, imm: i32) -> &mut Self

Opcode 120: store_ind_u8 (store u8 from rD to [rA + imm])

pub fn store_ind_u32(&mut self, rd: Reg, ra: Reg, imm: i32) -> &mut Self

Opcode 122: store_ind_u32 (store u32 from rD to [rA + imm])

pub fn store_ind_u64(&mut self, rd: Reg, ra: Reg, imm: i32) -> &mut Self

Opcode 123: store_ind_u64 (store u64 from rD to [rA + imm])

pub fn add_imm_32(&mut self, rd: Reg, ra: Reg, imm: i32) -> &mut Self

Opcode 131: add_imm_32 (rD = rA + imm, 32-bit)

pub fn add_imm_64(&mut self, rd: Reg, ra: Reg, imm: i32) -> &mut Self

Opcode 149: add_imm_64 (rD = rA + imm, 64-bit)

pub fn add_64(&mut self, rd: Reg, ra: Reg, rb: Reg) -> &mut Self

Opcode 200: add_64 (rD = rA + rB)

pub fn sub_64(&mut self, rd: Reg, ra: Reg, rb: Reg) -> &mut Self

Opcode 201: sub_64 (rD = rA - rB)

pub fn emit_raw(&mut self, byte: u8, is_instruction_start: bool)

Emit a raw byte with bitmask control.